
/***************************************************************
 ************************** STOP !! ****************************
 ***************************************************************/

/*
   If you are looking at this file, it should only be for
   educational purposes.  If you want a code base to start
   from to use in your own program, use the CLIENT source
   code, not this code.  This code implements a server that
   is essentially just a pipe between two game programs.
 */

/* Name : gamemanager.c
 *
 * Revision 1.4 for Nevo 1999/10/29 by kjm
 *
 * Revision 1.32 prints the move if an invalid move is returned
 * Revision 1.31 correctly check for ValidMove return code
 * Revision 1.3 for Upthrust 1998/10/16 by kjm
 *
 * Notes:  Based upon quixo.cc manager by idr
 * Revision 1.2  1996/10/08 19:47:12  idr
 * Revision 1.1  1996/10/07 22:24:59  idr
 * Initial revision
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include "nevo.h"

#define PLAYERHUMAN '-'
#define PLAYER1 'W'
#define PLAYER2 'B'

/****************/
play_board arrboard[10];
int Wcount[10];
int Bcount[10];
int Wdist[10];
int Bdist[10];

char comp_player;
int stopnow;
int stoptime;
/****************/

void ShowUsage( const char * name );
void StartProgram( const char * program, FILE * fd[], const char * flags);
void hit_key();
void new_init();
void display_board(int b);
void game_init();


/*****************************************************************************/
/* Name : main()
 *
 * Notes:
 *
 */

int main( int argc, char ** argv )
{
    int player1, player2;
    FILE *fd_player1[2], *fd_player2[2];
    int iWinner;
    int x,y;
    char whose_turn='W';
    struct move_piece mov;
    int look;
    char move[4];
    int validmove;
    int iWG;

    if ( argc < 3 )
    {
        ShowUsage( argv[ 0 ] );
        exit( 1 );
    }

    /* Create the structures for talking to our two players. */
    if (!strcmp("-",argv[1])) {
	player1 = PLAYERHUMAN;
    }
    else {
        player1 = PLAYER1;
    	StartProgram(argv[1], fd_player1, "W");
    }
    if (!strcmp("-",argv[2])) {
	player2 = PLAYERHUMAN;
    }
    else {
        player2 = PLAYER2;
    	StartProgram(argv[2], fd_player2, "B");
    }

    /* Initialize our board and the UI */
    srand(getpid());
    new_init();
    initscr();
    scrollok(stdscr,TRUE);
    display_board(0);
    while(1)
    {
     mov.oldx=-1; mov.oldy=-1; mov.newx=-1; mov.newy=-1;
     if ((whose_turn=='W') && (player1==PLAYERHUMAN)) {
       get_move_key(whose_turn,&mov); 
     }
     else if ((whose_turn=='B') && (player2==PLAYERHUMAN)) {
       get_move_key(whose_turn,&mov); 
     }
     else if (whose_turn=='W') {
       	printw("Waiting for computer player %c's move.\n",whose_turn);
	fscanf(fd_player1[0],"%d %d %d %d",
		&(mov.oldx), &(mov.oldy), &(mov.newx), &(mov.newy));
	refresh();
     }
     else if (whose_turn=='B') {
       	printw("Waiting for computer player %c's move.\n",whose_turn);
	fscanf(fd_player2[0],"%d %d %d %d",
		&(mov.oldx), &(mov.oldy), &(mov.newx), &(mov.newy));
	refresh();
     }
     validmove=ApplyMove(&mov,0);
     if (validmove==-1) {
	printw("%s entered an invalid move and loses! ",
		(whose_turn=='W')?"WHITE":"BLACK");	
        printw("The move was to: %d %d %d %d",
		mov.oldx,mov.oldy,mov.newx,mov.newy);
	refresh();
        hit_key();
        exit(0);
     }
     display_board(0);
     printw("The last move was : %d %d %d %d\n",
		mov.oldx,mov.oldy,mov.newx,mov.newy);
     refresh();
     iWG = WinGame(whose_turn,0);
     if (iWG==1)
     {
      printw("%s has Won!!!!!!!\n",(whose_turn=='W')?"WHITE":"BLACK");
      refresh();
      hit_key();
      exit(0);
     }
     else if (iWG==-1) {
      printw("%s has LOST all its pieces!\n",(whose_turn=='W')?"WHITE":"BLACK");
      refresh();
      hit_key();
      exit(0);
     }
     whose_turn=(whose_turn=='W')?'B':'W';
     /* Tell the new player what the last move was if its a computer */
     if ((whose_turn=='W') && (player1!=PLAYERHUMAN)) {
	fprintf(fd_player1[1],"%d %d %d %d\n", 
		mov.oldx,mov.oldy,mov.newx,mov.newy);
       	fflush(fd_player1[1]);
     }
     else if ((whose_turn=='B') && (player2!=PLAYERHUMAN)) {
	fprintf(fd_player2[1],"%d %d %d %d\n", 
		mov.oldx,mov.oldy,mov.newx,mov.newy);
       	fflush(fd_player2[1]);
     }
    }

}


/*****************************************************************************/
/* Name : StartProgram()
 *
 * Notes: This function starts a new process and runs the named program.  It
 *        fills fd[0] with a file handle that is tied to the new program's
 *        STDOUT (i.e., you can read the output of the child program through
 *        this file), and fills fd[1] with a file handle that is tied to the
 *        new program's STDIN.
 */

void StartProgram( const char * program, FILE* fd[], const char * player )
{
    int     pipeIn[ 2 ];        /* Provides STDIN for the CHILD.    */
    int     pipeOut[ 2 ];       /* Provides STDOUT for the CHILD.   */

    /* First, create the two pipes that we will use to communicate with the
     * child processes.
     */

    if ( pipe( pipeIn ) == -1 )
    {
        perror( "pipe( pipeIn )" );
        exit( 1 );
    }

    if ( pipe( pipeOut ) == -1 )
    {
        perror( "pipe( pipeOut )" );
        exit( 1 );
    }

    /* Create the child process! */
    if ( fork() == 0 )
    {
        /* Close the default STDIN and STDOUT file handles.  We want our
         * STDIN and STDOUT tied directly to our parent process.
         */

        close( STDIN_FILENO );
        close( STDOUT_FILENO );


        /* Close the write part of our STDIN pipe and the read part of our
         * STDOUT pipe.  As the child, we can't use these.
         */

        close( pipeIn[ 1 ] );
        close( pipeOut[ 0 ] );



        if ( dup2( pipeIn[ 0 ], STDIN_FILENO ) == -1 )
        {
            perror( "dup2( pipeIn[ 0 ], STDIN_FILENO )" );
            exit( 1 );
        }

        if ( dup2( pipeOut[ 1 ], STDOUT_FILENO ) == -1 )
        {
            perror( "dup2( pipeOut[ 1 ], STDOUT_FILENO )" );
            exit( 1 );
        }


        /* Run the specified program. */

        execl( program, program, player, NULL );


        /* If we get to this point, it means that execl() failed.  Let's
         * print an error message and exit.
         */

        perror( "execl()" );
        exit( 1 );
    }

    /* Close the read part of the child's STDIN pipe and the out part of
     * the child's STDOUT pipe.  As the parent, we (can't) won't be using
     * these parts.
     */

    close( pipeIn[ 0 ] );
    close( pipeOut[ 1 ] );

    /* Convert the parts of the pipes that we will be using to normal
     * stdio style FILEs.
     */

    fd[ 0 ] = fdopen( pipeOut[ 0 ], "r" );
    if ( fd[ 0 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }

    fd[ 1 ] = fdopen( pipeIn[ 1 ], "w" );
    if ( fd[ 1 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }
}




/*****************************************************************************/
/* Name : ShowUsage()
 *
 * Notes: This function attempts to show the user how to properly run our
 *        little program.
 */

void ShowUsage( const char * name )
{
    fprintf( stderr, "\nUsage: %s player1_program player2_program\n\n" \
                 "If the player_program is \"-\", then the player\n " \
                 "is a human player. Player1 is white, Player 2 black.\n",
             name );
    fprintf(stderr,"\nFor example, to have player 1 be \"myclient\"\n");
    fprintf(stderr,"and player 2 a human, you would invoke the manager as:\n");
    fprintf(stderr,"     %s myclient -\n\n",name);
}


void new_init()
 {
  FILE *fp;
  int i,j;
 
  fp=fopen("start.game","r");
  for(i=0;i<7;i++)
   {
    for(j=0;j<12;j++)
      if((arrboard[0][j][i]=fgetc(fp))=='-') arrboard[0][j][i]=' ';
    fgetc(fp);
   }
  fclose(fp);
  Wcount[0] = Bcount[0] = 12;
  Wdist[0] = Bdist[0] = 12;
 }

/******************************************************************/
/*                Procedures to print the board                   */
/******************************************************************/
void display_board(int b)
 {
  int i,j;
 
  clear();
  move(0,1);
  printw("     *   *     *   * 1 1"); 
  move(1,1);
  printw(" 0 1 2 3 4 5 6 7 8 9 0 1");
  move(0,40); printw("White: %d",ScoreBoard('W',b));
  move(1,40); printw("Black: %d",ScoreBoard('B',b));
  move(2,1);
  printw("-------------------------\n");
  for(i=0;i<7;i++)
   {
    printw("%d|",i);
    for(j=0;j<12;j++)
      printw("%c|",arrboard[b][j][i]);
    printw("\n -------------------------\n");
   }
  refresh();
 }

void hit_key()
 {
  printw("\nHit enter to continue.\n");
  refresh();
  getchar();
  refresh();
 }

/******************************************************************/
/*                           MAIN                                 */
/******************************************************************/
void game_init()
 {
 }


